
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "classes.h"

class locator {
public:
	locator();
	locator(const c3ga &xf);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);

	void set_xf(const c3ga &xf); ///< sets m_xf (and m_xfi)

	inline const c3ga &get_xf() const {return m_xf;} ///< returns m_xf
	inline const c3ga &get_xfi() const {return m_xfi;} ///< returns m_xfi

	/// returns position of locator as unit flat point
	inline GAIM_RETURN_TYPE get_position() const { 
		/*GAIM_RETURN_VAR(result);
		result = (get_xf() * (c3ga::no ^ c3ga::ni) * get_xfi())(GRADE2);
		return result;*/
		return apply_xf_to(c3ga::no ^ c3ga::ni);
	}

	/// transform a versor \param V according to this transfomation:
	inline GAIM_RETURN_TYPE apply_xf_to(const c3ga &V) const { 
		GAIM_RETURN_VAR(result);
		int g = V.gradeUsage();
		result = (get_xf() * V * get_xfi())(g);
		return result;
	}

	/// transform a versor \param V according to this transfomation:
	inline GAIM_RETURN_TYPE apply_xfi_to(const c3ga &V) const { 
		GAIM_RETURN_VAR(result);
		int g = V.gradeUsage();
		result = (get_xfi() * V * get_xf())(g);
		return result;
	}

	void pre_mul_xf(const c3ga &xf); ///< m_xf = xf * m_xf
	void post_mul_xf(const c3ga &xf); ///< m_xf = m_xf * xf;


	virtual bool can_scale() const; // should return true if this locator can scale
	// todo: 
/*	void addTransform(c3ga &tf);
	void setIdentityTransform();*/

	/// ,multiplies the transform with the current GL matrix
	void apply_transform_OpenGL() const;
//	void applyInverseTransformOpenGL();

/*	void applyTransformOpenGL(c3ga &tf);*/

	bool get_selected() const {return m_selected;}
	void set_selected(bool s) {m_selected = s;}

	/// returns just the translation part of the transform
	c3ga get_translator() const; 

	/// Turns this object into a string (e.g., for storing a scene to file)
	std::string to_string() const;

private:
	// todo: keep inverse transform in G2 version?
	c3ga m_xf, m_xfi;	 // transform & inverse transform (CGA versor)

	bool m_selected; /// set to true when this locator is selected by the UI
};


#endif /* _LOCATOR_H_ */

