
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>
#include "ui_versors.h"

c3ga create_camera_translation_versor_parallel(const c3ga &mouse_motion) {
    /*
    Exponentiate the free vector to get the translation versor:
    */
    return (0.5 * (mouse_motion ^ c3ga::ni)).exp();
}

c3ga create_camera_translation_versor_orthogonal(const c3ga &mouse_motion) {	
    /*
    Same code as in create_translation_versor_parallel,
    except we replaced mouse_motion with 
    ((mouse_motion % e2) * e3):
    */
    c3ga mouse_motion_orthogonal_camera_plane =
		(((mouse_motion % c3ga::e2) * c3ga::e3) ^ c3ga::ni);

    /*
    We can now directly exponentiate this free vector
    to get our translation versor:
    */
    return mouse_motion_orthogonal_camera_plane.exp();
}

c3ga create_camera_rotation_versor_outside_screen_plane(const c3ga &mouse_motion) {
	return (c3ga::e3 ^ mouse_motion).exp();
}

c3ga create_camera_rotation_versor_in_screen_plane(const c3ga &mouse_position, const c3ga &mouse_motion) {
	return (mouse_motion ^ mouse_position).exp();
}

c3ga create_translation_versor_orthogonal(const c3ga &camera_xf, const c3ga &locator_xf, 
										  const c3ga &mouse_motion) {
	/*
	We want the translation to become larger as the locator is further away
	from the camera, so first we compute the distance from the camera to
	the locator
	*/
	// compute origin of locator relative to camera
	c3ga relative_no = locator_xf * camera_xf.inverse() * c3ga::no * camera_xf * locator_xf.inverse();

	// compute distance from camera to locator
	GAIM_FLOAT dis = sqrt(-2.0 * ((relative_no << c3ga::no) / -(relative_no << c3ga::ni)).scalar());

	// compute translator in camera frame:
	return camera_xf * (-0.005 * dis * (mouse_motion % c3ga::e2) * c3ga::e3 * c3ga::ni).exp() * camera_xf.inverse();
}

c3ga create_translation_versor_parallel(
		const c3ga &camera_xf, 
		const c3ga &scaled_mouse_motion_in_camera_frame) {
	/*
	Compute translator in camera frame.
	scaled_mouse_motion_in_camera_frame is already 'ready for use' except that
	it is in the wrong frame.

    So we compute the free vector that represents the
    mouse motion, transform that to global coordinates,
    and then compute the exponent of that:
	*/
	return (0.5 * camera_xf * (c3ga::ni ^ scaled_mouse_motion_in_camera_frame)* camera_xf.inverse()).exp();
}

/*c3ga create_scaling_versor(const c3ga &locator_xf, const c3ga &mouse_motion) {
	//printf("Wha: %s\n", (0.01 * (mouse_motion % c3ga::e2) * (c3ga::no ^ c3ga::ni)).exp().string());
	return locator_xf * (0.01 * (mouse_motion % c3ga::e2) * (c3ga::no ^ c3ga::ni)).exp() * locator_xf.inverse();
}*/

c3ga create_scaling_versor(const c3ga &mouse_motion) {
	return (0.01 * (mouse_motion % c3ga::e2) * (c3ga::no ^ c3ga::ni)).exp();
}
