
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "vertex.h"

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>


vertex::vertex() {
}

vertex::vertex(const c3ga &pt, const c3ga &att, const c3ga &tex_pt) :
	m_pt(pt), m_att(att), m_tex_pt(tex_pt) {
}

vertex::vertex(const vertex &v) :
	m_pt(v.m_pt), m_att(v.m_att), m_tex_pt(v.m_tex_pt) {
}

vertex::~vertex() {
}

vertex &vertex::operator=(const vertex &v) {
	if (this != &v) {
		m_pt = v.m_pt;
		m_att = v.m_att;
		m_tex_pt = v.m_tex_pt;
	}
	return *this;
}

/// calls glVertex() for OpenGL rendering
void vertex::glVertex() const {
	// convert regular point in flat point representation:
	c3ga fpt(get_pt() ^ c3ga::ni);

	// get the coordinates of the point:
	const GAIM_FLOAT *c = fpt[GRADE2];
/*	printf("%f %f %f %f\n", (GLfloat)c[C3GA_E1_NI], 
		(GLfloat)c[C3GA_E2_NI], 
		(GLfloat)c[C3GA_E3_NI], 
		(GLfloat)c[C3GA_NO_NI]);*/

	// call OpenGL to specify vertex coordinates:
	glVertex4f(
		(GLfloat)c[C3GA_E1_NI], 
		(GLfloat)c[C3GA_E2_NI], 
		(GLfloat)c[C3GA_E3_NI], 
		(GLfloat)c[C3GA_NO_NI]);
}

/// calls glNormal() for OpenGL rendering
void vertex::glNormal() const {
	// get the local surface normal (dual of the attitude)
	c3ga normal(-get_att().dual());

	// get the coordinates of the normal:
	const GAIM_FLOAT *c = normal[GRADE2];

	float n[3] = {
		(GLfloat)c[C3GA_E1_NI], 
		(GLfloat)c[C3GA_E2_NI], 
		(GLfloat)c[C3GA_E3_NI]};

	// call OpenGL to specify normal coordinates:
	glNormal3f(
		(GLfloat)c[C3GA_E1_NI], 
		(GLfloat)c[C3GA_E2_NI], 
		(GLfloat)c[C3GA_E3_NI]);
}
