
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _BSP_NODE_H_
#define _BSP_NODE_H_

#include <vector>
#include "classes.h"
#include "face.h"

class bsp_descent_info {
public:
	inline bsp_descent_info(const mesh *_m, const c3ga::normalizedFlatPoint &_fpt1, c3ga::mv::Float _d1, const c3ga::normalizedFlatPoint &_fpt2, c3ga::mv::Float _d2, 
		const c3ga::dualPlane &_ray_dual_plane, const c3ga::dualLine &_ray_dual_line, surface_point &_spt, bool _find_closest) :
		m(_m),
		fpt1(_fpt1), d1(_d1),
		fpt2(_fpt2), d2(_d2),
		ray_dual_plane(_ray_dual_plane),
		ray_dual_line(_ray_dual_line),
		spt(_spt),
		find_closest(_find_closest) {
	}

	c3ga::normalizedFlatPoint fpt1;
	c3ga::mv::Float d1;
	c3ga::normalizedFlatPoint fpt2;
	c3ga::mv::Float d2;

	const mesh *m;
	const c3ga::dualPlane &ray_dual_plane;
	const c3ga::dualLine &ray_dual_line;
	
	surface_point &spt;
	bool find_closest;
};

class bsp_node {
public:
	bsp_node(); ///< null constructor
	bsp_node(const std::vector<int> &face); ///< leaf node constructor
	bsp_node(const c3ga::plane &pl, bsp_node *backside, bsp_node *frontside); ///< generic node constructor
	bsp_node(const bsp_node &b); ///< copy constructor
	~bsp_node(); ///< destructor

	/// Copy constructor
	bsp_node &operator=(const bsp_node &b);

	/**
	docs todo
	*/
	inline bool find_intersection(bsp_descent_info &di) const {
		if (m_frontside_node && m_backside_node) // this is a partition node
			return find_intersection_internal(di);
		else // this is a leaf node, so search all faces
			return find_intersection_leaf(di);
	}

	/// Visits the bsp nodes that contain fpt (testing function)
	void visit_bsp_node(const mesh *m, const c3ga::normalizedFlatPoint &fpt) const ;

	/// Inserts face \param idx into this BSP tree, given the (possibly clipped) vertices of the face
	void insert_face(int face_idx, const std::vector<c3ga::normalizedFlatPoint> &vertex);


private:
	c3ga::plane m_pl; ///< the partition plane (CGA plane)

	/**
	The bsp node for the partition of the space that lies on the 'backside' of m_pl
	 (destruction of object managed by this class)
	*/
	bsp_node *m_backside_node; 
	/**
	The bsp node for the partition of the space that lies on the 'frontside' of m_pl
	 (destruction of object managed by this class)
	*/
	bsp_node *m_frontside_node;
	

	/// Finds intersection for leaf nodes
	bool find_intersection_leaf(bsp_descent_info &di) const;
	/// Finds intersection for internal nodes
	bool find_intersection_internal(bsp_descent_info &di) const;

	std::vector<int> m_face; ///< the indices of the faces that ended up in this node (only is this is a leave node)
};

#endif /* _BSP_NODE_H_ */

