
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


/*
Command line options:
-r : just render
-s .... load scene '....'

*/
#include <stdio.h>
#include <math.h>
#include <string>

#include "../shared.src/pngfe.h"
#include "../shared.src/mt19937-2.h"
#include "../shared.src/fltk_stuff.h"
#include "../shared.src/timing.h"

#include "scene.h"
#include "c3ga_util.h"
#include "version.h"

int main(int argc, char *argv[]) {

	bool interactive = false; // no UI for this version 
	std::string sceneFilename = "../files/my_fifth_scene.rsn"; // default scene name

	// parse simple command line options
	for (int i = 1; i < argc; i++) {
		if (argv[i] == NULL) continue;
		else if (std::string(argv[i]) == "-r") {// just render
			argv[i] = "";
			interactive = false;
		}
		else if ((i < (argc-1)) && (std::string(argv[i]) == "-s")) {//  load scene [i+1]
			sceneFilename = argv[i+1];
			argv[i] = argv[i+1] = NULL;
		}
	}

	// remove used command line options:
	int j = 0;
	for (int i = 0; i < argc; i++) {
		if (argv[i]) argv[j++] = argv[i];
	}
	argc = j;

	// check sanity of command line arguments
	if ((interactive == false) && (sceneFilename.size() == 0)) {
		printf("Need a scene filename to render. Use:\n"
			"%s -r -s filename.rsn\n", argv[0]);
		return -1;
	}

	scene S;

	// load scene, if any . . .
	if (sceneFilename.size()) {
		printf("Loading scene %s\n", sceneFilename.c_str());
		S.load_rsn(sceneFilename);
	}



	// just render:
	const int w = 512, h = 512;
	std::vector<unsigned char>image(w * h * 3);
    u_timeElapsed();
	e3ga::g2Profiling::reset();
	S.render(0, &(image[0]), w, h, 1, 0, NULL, 4);
	e3ga::g2Profiling::save();
	printf("%f secs\n", u_timeElapsed());

	// save result
	{
		std::string imageFilename;
		std::string extension = std::string("_") + g_ray_version + ".png";
		if (sceneFilename.substr(sceneFilename.size()-4) == ".rsn")
			imageFilename = sceneFilename.substr(0, sceneFilename.size()-4) + extension;
		else imageFilename = sceneFilename + extension;

		printf("Saving to image %s\n", imageFilename.c_str());
		save_png(imageFilename.c_str(), &(image[0]), w, h);
	}
	
	return 0;
}
