
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _C3GA_UTIL_H_
#define _C3GA_UTIL_H_

#include <math.h>
#include "c3ga.h"
#include "e3ga.h"

/// Returns a point at the location specified by 3D Euclidean vector \param p
inline c3ga cga_point(const c3ga &p) {
	return p + c3ga::no + 0.5f * p.norm_a() * c3ga::ni;
}

/**
Returns the Euclidean distance between two points, squared.
Assumes the points are 'normalized'
*/
inline GAIM_FLOAT flat_point_distance_squared(const c3ga &fpt1, const c3ga &fpt2) {
	return (fpt1 - fpt2).norm_a();
}

/**
Returns the Euclidean distance between two points.
Assumes the points are 'normalized'
*/
inline GAIM_FLOAT flat_point_distance(const c3ga &fpt1, const c3ga &fpt2) {
	return sqrt(flat_point_distance_squared(fpt1, fpt2));
}


/// dissects a point pair \param pp into two CGA points \param p1 and \param p2 .
void dissect_point_pair(const c3ga &pp, c3ga &p1, c3ga &p2);

/// dissects a point pair \param pp into two flat points \param fp1 and \param fp2 .
//void dissect_point_pair_flat(const c3ga &pp, c3ga &fp1, c3ga &fp2);
/// dissects a point pair \param pp into two normalized flat points \param fp1 and \param fp2 .
void dissect_point_pair_normalized_flat(const c3ga &pp, c3ga &fpt1, c3ga &fpt2);


/**
Returns a unit versor according to the 'reverse' norm (B . ~B).
Returns 0 for null versor (B . ~B) == 0
*/
inline GAIM_RETURN_TYPE unit_r(const c3ga &x) {
	GAIM_RETURN_VAR(result);

	GAIM_FLOAT n = x.norm_b();
	result = (n < 0.0f) ? x / sqrt(-n) :
		((n > 0.0f) ? x / sqrt(n) : 0.0f);

	return result;
}

/**
Returns a unit versor according to the 'euclidean' norm 
(euclidean_lift(B) . ~euclidean_lift(B)).
*/
inline GAIM_RETURN_TYPE unit_e(const c3ga &x) {
	GAIM_RETURN_VAR(result);

	GAIM_FLOAT n = x.norm_a();
	result = (n == 0.0f) ? 0.0f : x / sqrt(n);

	return result;
}

/**
Converts a conformal 3D TSR (translate, scale, rotate) transformation 
versor into a 3D 'linear algebra' translate, scale, rotate (3x3 matrix) sequence. Results 
are returned in \param T , \param S and \param R .
*/
void conformal_TSR_versor_to_3DGA(const c3ga &TSR, e3ga::vector &T, GAIM_FLOAT &S, e3ga::rotor &R);


#endif /* #ifndef _C3GA_UTIL_H_ */
