
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/*
This file contains rather dull and ancient code to parse .rtm files.
*/

#include "scene.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "c3ga.h"
#include "c3ga_util.h"

#include "../shared.src/fltk_stuff.h"
#include "../shared.src/string_util.h"

/*
This file contains rather dull and ancient code to parse .rsn (ray tracer scene) files.
*/


// loads scene from filename
void scene::load_rsn(const std::string &filename) {
	m_light_source.clear();
//	m_mesh.clear(); 
	m_current_camera = 0;
	m_camera.clear();
	m_camera.resize(2);
	m_model.clear();
	m_rtm_model_cache.clear();
	m_rtm_mesh_cache.clear();

	const int maxLineLength = 512;
	char line[maxLineLength];
	int lineNumber = 0, nTabs, l;
	int err = 0;
	FILE *F;
	char entryname[256];
	int i1;
	float f1;
	float red, green, blue;

	const int path_len = 1024;
	char s1[path_len], s2[path_len];
	char path[path_len];
	path_from_filename(path, path_len, filename.c_str());

	int nbCameras = 0;

	// load scene data into these objects before adding them to the scene:
	model Model;
	light_source Light_source;
	camera Camera;
	
	// helper class for multivector parser
	c3ga_ben cBen('[', ']');

	if ( (F = fopen(filename.c_str(), "rb")) == NULL) {
		throw "scene::load_rtm(): can not open '" + filename + "'";
	}

	while (!feof(F)) {
		if ( (l = read_line(F, line, maxLineLength, &nTabs)) < 0) {
			break;
		}
		lineNumber++;
		if (empty_line(line)) continue;

		if (sscanf(line, "%s ", entryname) != 1) continue;


/***************************** MODEL BEGIN ***********************************/

		if (!strcmpnc(entryname, "model_begin")) {		// model_begin
			Model = model();
		}
		else if (!strcmpnc(entryname, "model_xf")) {		// model_xf
			char *mv_str = line + strlen("model_xf") + 1;
			c3ga mv;
			if ((mv.parseString(mv_str, &cBen)) < 0) {
				err = -1; break;
			}
			else Model.set_xf(mv);
		}
		else if (!strcmpnc(entryname, "mesh")) {		// mesh
			if (err = read_filename(line, s1)) break;
			fl_filename_absolute_to(s2, path_len, s1, path);
			{
				try {
					const model &tmp_model(get_rtm(s2));
					Model.set_mesh(tmp_model.get_mesh());
				} catch(const std::string &) {
					//add_error(S); // todo! (and display errors after loading!)
				}
			}
		}

		else if (!strcmpnc(entryname, "model_ambient_color")) {		// model_ambient_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Model.set_ambient_color(color(red, green, blue));
		}

		else if (!strcmpnc(entryname, "model_diffuse_color")) {		// model_diffuse_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Model.set_diffuse_color(color(red, green, blue));
		}

		else if (!strcmpnc(entryname, "model_specular_color")) {		// model_specular_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Model.set_specular_color(color(red, green, blue));
		}

		else if ((!strcmpnc(entryname, "ambient_texture")) ||
			(!strcmpnc(entryname, "diffuse_texture")) ||
			(!strcmpnc(entryname, "specular_texture")) ||
			(!strcmpnc(entryname, "shininess_texture")) ||
			(!strcmpnc(entryname, "bump_map_texture"))) {		// XXX_texture
			if (err = read_filename(line, s1)) break;
			if (strlen(s1)) {
				fl_filename_absolute_to(s2, path_len, s1, path);
				texture &tex(Model.get_texture(entryname));
				tex.load(s2);
				if (tex.valid()) tex.set_enabled(true);
			}
		}

		else if ((!strcmpnc(entryname, "ambient_texture_enabled")) ||
			(!strcmpnc(entryname, "diffuse_texture_enabled")) ||
			(!strcmpnc(entryname, "specular_texture_enabled")) ||
			(!strcmpnc(entryname, "shininess_texture_enabled")) ||
			(!strcmpnc(entryname, "bump_map_texture_enabled")) ) 	{		// ambient_texture_enabled
			if ((sscanf(line, "%s %s", entryname, s1) != 2)) {err = -1;break;	}
			Model.set_texture_enabled(entryname, !strcmpnc(s1, "true"));
		}

		else if (!strcmpnc(entryname, "shininess")) {		// shininess
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_shininess(f1);
		}
		else if (!strcmpnc(entryname, "bumpyness")) {		// bumpyness
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_bumpyness(f1); 
		}

		else if (!strcmpnc(entryname, "direct_lighting_factor")) {		// direct_lighting_factor
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_direct_lighting_factor(f1);
		}
		else if (!strcmpnc(entryname, "reflection_factor")) {		// reflection_factor
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_reflection_factor(f1);
		}
		else if (!strcmpnc(entryname, "refraction_factor")) {		// refraction_factor
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_refraction_factor(f1);
		}
		else if (!strcmpnc(entryname, "refractive_index")) {		// refractive_index
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Model.set_refractive_index(f1);
		}        
		else if (!strcmpnc(entryname, "model_end")) {		// model_end
			// check if a mesh is present, otherwise don't add to scene
			if (Model.get_mesh())
				add_model(Model);
			Model = model();
		}
/***************************** MODEL END ***********************************/



/***************************** LIGHT SOURCE BEGIN ***********************************/
		else if (!strcmpnc(entryname, "light_source_begin")) {		// light_source_begin
			Light_source = light_source();
		}
		else if (!strcmpnc(entryname, "light_source_xf")) {		// light_source_xf
			char *mv_str = line + strlen("light_source_xf") + 1;
			c3ga mv;
			if ((mv.parseString(mv_str, &cBen)) < 0) {
				err = -1; break;
			}
			else Light_source.set_xf(mv);
		}
		else if (!strcmpnc(entryname, "light_source_ambient_color")) {		// light_source_ambient_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Light_source.set_ambient_color(color(red, green, blue));
		}

		else if (!strcmpnc(entryname, "light_source_diffuse_color")) {		// light_source_diffuse_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Light_source.set_diffuse_color(color(red, green, blue));
		}

		else if (!strcmpnc(entryname, "light_source_specular_color")) {		// light_source_specular_color
			if ((sscanf(line, "%s %f %f %f", entryname, &red, &green, &blue) != 4)) {err = -1;break;	}
			Light_source.set_specular_color(color(red, green, blue));
		}
		else if (!strcmpnc(entryname, "spot_cutoff")) {		// spot_cutoff
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Light_source.set_spot_cutoff(f1);
		}        
		else if (!strcmpnc(entryname, "spot_exponent")) {		// spot_exponent
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Light_source.set_spot_exponent(f1);
		}        
		else if (!strcmpnc(entryname, "constant_attenuation")) {		// constant_attenuation
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Light_source.set_constant_attenuation(f1);
		}        
		else if (!strcmpnc(entryname, "linear_attenuation")) {		// linear_attenuation
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Light_source.set_linear_attenuation(f1);
		}        
		else if (!strcmpnc(entryname, "direct_lighting_factor")) {		// quadratic_attenuation
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Light_source.set_quadratic_attenuation(f1);
		}        

		else if (!strcmpnc(entryname, "light_source_end")) {		// light_source_end
			// check if a mesh is present, otherwise don't add to scene
			add_light_source(Light_source);
			Light_source = light_source();
		}
/***************************** LIGHT SOURCE END ***********************************/

		else if (!strcmpnc(entryname, "display_lights")) {		// display_lights
			if ((sscanf(line, "%s %d", entryname, &i1) != 2)) {err = -1;break;	}
			try {
				set_display_lights(i1 != 0);
			} catch (const std::string &) {
					//add_error(S); // todo! (and display errors after loading!)
			}
		}

/***************************** CAMERA BEGIN ***********************************/
		else if (!strcmpnc(entryname, "camera_begin")) {		// camera_begin
			Camera = camera();
		}
		else if (!strcmpnc(entryname, "camera_xf")) {		// camera_xf
			char *mv_str = line + strlen("camera_xf") + 1;
			c3ga mv;
			if ((mv.parseString(mv_str, &cBen)) < 0) {
				err = -1; break;
			}
			else Camera.set_xf(mv);
		}
		else if (!strcmpnc(entryname, "camera_fov")) {		// camera_fov
			if ((sscanf(line, "%s %f", entryname, &f1) != 2)) {err = -1;break;	}
			Camera.set_fov_width(f1);
		}
		else if (!strcmpnc(entryname, "camera_end")) {		// camera_end
			// check if a mesh is present, otherwise don't add to scene
			try {
				set_camera(nbCameras++, Camera);
			} catch (const std::string &) {
					//add_error(S); // todo! (and display errors after loading!)
			}


			Camera = camera();
		}
/***************************** CAMERA END ***********************************/


		else if (!strcmpnc(entryname, "current_camera")) {		// current_camera
			if ((sscanf(line, "%s %d", entryname, &i1) != 2)) {err = -1;break;	}
			try {
				set_current_camera(i1);
			} catch (const std::string &) {
					//add_error(S); // todo! (and display errors after loading!)
			}
		}


		/* ... */

		else {
			//printf("warning: scene::load_rsn(): Unknown entry '%s' at line '%d'\n", entryname, lineNumber);
		}
	}

	// todo: check if all textures were loaded correctly . . .


	if (err) {
		// error detected:
		char buf[1024];
		sprintf(buf, "scene::load_rsn(): Invalid entry '%s' at line %d\n", entryname, lineNumber);
		throw std::string(buf);
	}

	fclose(F);
}



