
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#include <GL/gl.h>

#include "c3ga_util.h"
#include "e3la.h"

void dissect_point_pair(const c3ga &pp, c3ga &pt1, c3ga &pt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	pt1 = v3 + v2;
	pt2 = v3 - v2;
}

void dissect_point_pair_normalized_flat(const c3ga &pp, c3ga &fpt1, c3ga &fpt2) {
	GAIM_FLOAT n = sqrt((pp << pp).scalar());
	c3ga v1(c3ga::ni << pp);
	c3ga v2(n * v1);
	c3ga v3(v1 << pp);

	GAIM_FLOAT scale = (GAIM_FLOAT)1.0 / ((v2 << c3ga::ni).scalar() + (v3 << c3ga::ni).scalar());
	fpt1 = (scale * c3ga::ni) ^ (v3 + v2);
	fpt2 = (scale * c3ga::ni) ^ (v3 - v2);
}

void conformal_TSR_versor_to_3DLA(const c3ga &TSR, e3la_vec &T, GAIM_FLOAT &S, e3la_mat &R) {
	// extract translation
	c3ga ifo= TSR * (c3ga::no ^ c3ga::ni) / TSR; // ifo =  image of flat origin
	T.set(ifo[GRADE2][C3GA_E1_NI], ifo[GRADE2][C3GA_E2_NI], ifo[GRADE2][C3GA_E3_NI]);

	// extract scaling
	c3ga ii = TSR * (c3ga::ni) / TSR; // ii = image of infinity
	S = ii[GRADE1][C3GA_NI];
	
	// extract rotation
	c3ga ie1 = TSR * c3ga::e1 / TSR; // ie1 = image of e1
	c3ga ie2 = TSR * c3ga::e2 / TSR; // ie2 = image of e2
	c3ga ie3 = TSR * c3ga::e3 / TSR; // ie3 = image of e3
	R.m11 = ie1[GRADE1][C3GA_E1]; R.m21 = ie1[GRADE1][C3GA_E2]; R.m31 = ie1[GRADE1][C3GA_E3];
	R.m12 = ie2[GRADE1][C3GA_E1]; R.m22 = ie2[GRADE1][C3GA_E2]; R.m32 = ie2[GRADE1][C3GA_E3];
	R.m13 = ie3[GRADE1][C3GA_E1]; R.m23 = ie3[GRADE1][C3GA_E2]; R.m33 = ie3[GRADE1][C3GA_E3];
}



