
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "c3ga.h"
#include "p3ga.h"

class locator {
public:
	locator();
	locator(const c3ga &TSR); ///< constructor from conformal TSR versor (uses conformal_TSR_versor_to_P3GA to convert)
	locator(const p3ga::om &tm, const p3ga::om &itm);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);


	void set_xf(const c3ga &TSR); ///< sets transform from conformal TSR versor (uses conformal_TSR_versor_to_3DLA to convert)

	inline p3ga::mv::Float get_scale() const {
		return m_tm.m_c[15];
	}

	inline p3ga::normalizedPoint get_translation() const {
		p3ga::point pt = p3ga::_point(apply_om(m_tm, p3ga::e0));
		return p3ga::_normalizedPoint(pt * (1.0f / _Float(pt & p3ga::e0)));
	}

	inline p3ga::vectorE3GA transform_direction(const p3ga::vectorE3GA &dir) const {
		return p3ga::_vectorE3GA(apply_om(m_tm, dir));
	}
	inline p3ga::vectorE3GA inverse_transform_direction(const p3ga::vectorE3GA &dir) const {
		return p3ga::_vectorE3GA(apply_om(m_itm, dir));
	}

	inline p3ga::normalizedPoint transform_point(const p3ga::normalizedPoint &dir) const {
		return p3ga::_normalizedPoint(apply_om(m_tm, dir));
	}
	inline p3ga::normalizedPoint inverse_transform_point(const p3ga::normalizedPoint &dir) const {
		return p3ga::_normalizedPoint(apply_om(m_itm, dir));
	}

	inline p3ga::line transform_line(const p3ga::line &dir) const {
		return p3ga::_line(apply_om(m_tm, dir));
	}
	inline p3ga::line inverse_transform_line(const p3ga::line &dir) const {
		return p3ga::_line(apply_om(m_itm, dir));
	}

	inline p3ga::plane transform_plane(const p3ga::plane &dir) const {
		return p3ga::_plane(apply_om(m_tm, dir));
	}
	inline p3ga::plane inverse_transform_plane(const p3ga::plane &dir) const {
		return p3ga::_plane(apply_om(m_itm, dir));
	}

protected:
	p3ga::om m_tm, m_itm;
};


#endif /* _LOCATOR_H_ */

