
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <string.h>
#if defined(WIN32) && !defined(__CYGWIN__)
# include <direct.h>
//# define getcwd(a,b) _getdcwd(0,a,b)
#else
# include <unistd.h>
# ifdef __EMX__
#  define getcwd _getcwd2
# endif
#endif

// disable some annoying warnings in VC++ due to FLTK (1.1.6)
#ifdef WIN32
#pragma warning( disable : 4311 4312 4244 )
#endif

#include <FL/Fl_File_Chooser.H>

#include "fltk_stuff.h"

namespace {
/*
This is the fl_file_chooser() code + some small support code, 
copied directly from FLTK 1.1.5rc2,  but with preview disabled, 
since it kept causing me problems with pngs.
*/
static Fl_File_Chooser	*fc = (Fl_File_Chooser *)0;
static void		(*current_callback)(const char*) = 0;
static void callback(Fl_File_Chooser *, void*) {
  if (current_callback && fc->value())
    (*current_callback)(fc->value());
}

size_t				/* O - Length of string */
fl_strlcpy(char       *dst,	/* O - Destination string */
           const char *src,	/* I - Source string */
	   size_t      size) {	/* I - Size of destination string buffer */
  size_t	srclen;		/* Length of source string */


 /*
  * Figure out how much room is needed...
  */

  size --;

  srclen = strlen(src);

 /*
  * Copy the appropriate amount...
  */

  if (srclen > size) srclen = size;

  memcpy(dst, src, srclen);
  dst[srclen] = '\0';

  return (srclen);
}

void fl_file_chooser_callback(void (*cb)(const char*)) {
  current_callback = cb;
}

}

char *					// O - Filename or NULL
fl_file_chooser_no_preview(const char *message,	// I - Message in titlebar
                const char *pat,	// I - Filename pattern
		const char *fname,	// I - Initial filename selection
		int        relative) {	// I - 0 for absolute path
  static char	retname[1024];		// Returned filename

  if (!fc) {
    if (!fname || !*fname) fname = ".";

    fc = new Fl_File_Chooser(fname, pat, Fl_File_Chooser::CREATE, message);
	fc->preview(false);
    fc->callback(callback, 0);
  } else {
	fc->preview(false);
    fc->type(Fl_File_Chooser::CREATE);
    fc->filter(pat);
    fc->label(message);

    if (!fname || !*fname) {
      if (fc->filter() != pat && (!pat || !fc->filter() ||
          strcmp(pat, fc->filter())) && fc->value()) {
	// if pattern is different, remove name but leave old directory:
	fl_strlcpy(retname, fc->value(), sizeof(retname));

	char *p = strrchr(retname, '/');

        if (p) {
	  // If the filename is "/foo", then the directory will be "/", not
	  // ""...
	  if (p == retname)
	    retname[1] = '\0';
	  else
	    *p = '\0';
	}

	// Set the directory...
	fc->directory(retname);
      }
    }
    else
      fc->value(fname);
  }

  fc->show();

  while (fc->shown())
    Fl::wait();

  if (fc->value() && relative) {
    fl_filename_relative(retname, sizeof(retname), fc->value());

    return retname;
  } else if (fc->value()) return (char *)fc->value();
  else return 0;
}


// frontend for fl_filename_relative() that allows it to work relative to any cwd
int					// O - 0 if no change, 1 if changed
fl_filename_relative_to(char       *to,	// O - Relative filename
                     int        tolen,	// I - Size of "to" buffer
                     const char *from, // I - Absolute filename
					 const char *relative_to) {
	const int l = 1023;
	char cwd[l+1];

	// get current path, so we can restore it later
	char *tmp;
#ifdef WIN32
	tmp = _getcwd(cwd, l);
#else 
	tmp = getcwd(cwd, l);
#endif
	if (!tmp) { // getcwd() failure
		// why can't FLTK just export strlcpy and such?
		tolen--;
		int l = (tolen < (int)strlen(from)) ? tolen : (int)strlen(from);
		memcpy(to, from, l);
		to[l] = 0;
		return 0;
	}

	// set current parh to 'relative_to'
#ifdef WIN32
	_chdir(relative_to);
#else 
	chdir(relative_to);
#endif

	// use the FLTK function to make the path 'from' relative to the cwd (== relative_to)
	int result = fl_filename_relative(to, tolen, from);

	// restore original path
#ifdef WIN32
	_chdir(cwd);
#else 
	chdir(cwd);
#endif

	return result;

}

int fl_filename_absolute_to(char *to, int tolen, const char *from, const char *relative_to) {
	const int l = 1023;
	char cwd[l+1];

	// get current path, so we can restore it later
	char *tmp;
#ifdef WIN32
	tmp = _getcwd(cwd, l);
#else 
	tmp = getcwd(cwd, l);
#endif
	if (!tmp) { // getcwd() failure
		// why can't FLTK just export strlcpy and such?
		tolen--;
		int l = (tolen < (int)strlen(from)) ? tolen : (int)strlen(from);
		memcpy(to, from, l);
		to[l] = 0;
		return 0;
	}

	// set current parh to 'relative_to'
#ifdef WIN32
	_chdir(relative_to);
#else 
	chdir(relative_to);
#endif

	// use the FLTK function to make the path 'from' relative to the cwd (== relative_to)
	int result = fl_filename_absolute(to, tolen, from);

	// restore original path
#ifdef WIN32
	_chdir(cwd);
#else 
	chdir(cwd);
#endif

	return result;

}

void path_from_filename(char *path, int pathlen, const char *filename) {
	const char *ptr = strrchr(filename, '/');
	const char *ptr_bsl = strrchr(filename, '\\');
	if (ptr_bsl > ptr) ptr = ptr_bsl;

	pathlen--;
	int l = ptr - filename;
	if (l > pathlen) l = pathlen;
	memcpy(path, filename, l);
	path[l] = 0;
}

